@@ -1,6 +1,12 @@
 use super::Handle;
 
 pub trait ErrorApi {
+    type ErrorApiImpl: ErrorApiImpl;
+
+    fn error_api_impl() -> Self::ErrorApiImpl;
+}
+
+pub trait ErrorApiImpl {
     fn signal_error(&self, message: &[u8]) -> !;
 
     fn signal_error_from_buffer(&self, message_handle: Handle) -> !;
@@ -9,9 +15,9 @@ pub trait ErrorApi {
 /// An error handler that simply panics whenever `signal_error` is called.
 /// Especially useful for unit tests.
 /// Implements `ErrorApi`.
-pub struct PanickingErrorApi;
+pub struct PanickingErrorApiImpl;
 
-impl ErrorApi for PanickingErrorApi {
+impl ErrorApiImpl for PanickingErrorApiImpl {
     fn signal_error(&self, message: &[u8]) -> ! {
         panic!(
             "PanickingErrorApi panicked: {}",
@@ -23,3 +29,16 @@ impl ErrorApi for PanickingErrorApi {
         panic!("PanickingErrorApi panicked via signal_error_from_buffer")
     }
 }
+
+/// An error handler that simply panics whenever `signal_error` is called.
+/// Especially useful for unit tests.
+/// Implements `ErrorApi`.
+pub struct PanickingErrorApi;
+
+impl ErrorApi for PanickingErrorApi {
+    type ErrorApiImpl = PanickingErrorApiImpl;
+
+    fn error_api_impl() -> Self::ErrorApiImpl {
+        PanickingErrorApiImpl
+    }
+}
