@@ -2,7 +2,6 @@ mod blockchain_wrapper;
 mod call_value_wrapper;
 mod crypto_wrapper;
 mod error_helper;
-mod managed_type_helper;
 mod print_helper;
 mod send_wrapper;
 mod serializer;
@@ -11,7 +10,6 @@ pub use blockchain_wrapper::BlockchainWrapper;
 pub use call_value_wrapper::CallValueWrapper;
 pub use crypto_wrapper::CryptoWrapper;
 pub use error_helper::ErrorHelper;
-pub use managed_type_helper::ManagedTypeHelper;
 pub use print_helper::PrintHelper;
 pub use send_wrapper::SendWrapper;
 pub use serializer::ManagedSerializer;
