@@ -1,7 +1,9 @@
+use core::marker::PhantomData;
+
 use elrond_codec::TryStaticCast;
 
 use crate::{
-    api::{EndpointFinishApi, ErrorApi, ManagedTypeApi},
+    api::{EndpointFinishApi, EndpointFinishApiImpl, ErrorApi, ErrorApiImpl, ManagedTypeApi},
     elrond_codec::{EncodeError, TopEncode, TopEncodeOutput},
     err_msg,
     types::{BigInt, BigUint, ManagedBuffer, ManagedBufferCachedBuilder, ManagedType},
@@ -9,37 +11,39 @@ use crate::{
 
 struct ApiOutputAdapter<FA>
 where
-    FA: ManagedTypeApi + EndpointFinishApi + Clone + 'static,
+    FA: ManagedTypeApi + EndpointFinishApi,
 {
-    api: FA,
+    _phantom: PhantomData<FA>,
 }
 
 impl<FA> ApiOutputAdapter<FA>
 where
-    FA: ManagedTypeApi + EndpointFinishApi + Clone + 'static,
+    FA: ManagedTypeApi + EndpointFinishApi,
 {
     #[inline]
-    fn new(api: FA) -> Self {
-        ApiOutputAdapter { api }
+    fn new() -> Self {
+        ApiOutputAdapter {
+            _phantom: PhantomData,
+        }
     }
 }
 
 impl<FA> TopEncodeOutput for ApiOutputAdapter<FA>
 where
-    FA: ManagedTypeApi + EndpointFinishApi + Clone + 'static,
+    FA: ManagedTypeApi + EndpointFinishApi,
 {
     type NestedBuffer = ManagedBufferCachedBuilder<FA>;
 
     fn set_slice_u8(self, bytes: &[u8]) {
-        self.api.finish_slice_u8(bytes);
+        FA::finish_api_impl().finish_slice_u8(bytes);
     }
 
     fn set_u64(self, value: u64) {
-        self.api.finish_u64(value);
+        FA::finish_api_impl().finish_u64(value);
     }
 
     fn set_i64(self, value: i64) {
-        self.api.finish_i64(value);
+        FA::finish_api_impl().finish_i64(value);
     }
 
     #[inline]
@@ -54,13 +58,13 @@ where
         F: FnOnce(Self) -> Result<(), EncodeError>,
     {
         if let Some(managed_buffer) = value.try_cast_ref::<ManagedBuffer<FA>>() {
-            self.api.finish_managed_buffer_raw(managed_buffer.handle);
+            FA::finish_api_impl().finish_managed_buffer_raw(managed_buffer.handle);
             Ok(())
         } else if let Some(big_uint) = value.try_cast_ref::<BigUint<FA>>() {
-            self.api.finish_big_uint_raw(big_uint.handle);
+            FA::finish_api_impl().finish_big_uint_raw(big_uint.handle);
             Ok(())
         } else if let Some(big_int) = value.try_cast_ref::<BigInt<FA>>() {
-            self.api.finish_big_int_raw(big_int.handle);
+            FA::finish_api_impl().finish_big_int_raw(big_int.handle);
             Ok(())
         } else {
             else_serialization(self)
@@ -72,8 +76,7 @@ where
     }
 
     fn finalize_nested_encode(self, nb: Self::NestedBuffer) {
-        self.api
-            .finish_managed_buffer_raw(nb.into_managed_buffer().get_raw_handle());
+        FA::finish_api_impl().finish_managed_buffer_raw(nb.into_managed_buffer().get_raw_handle());
     }
 }
 
@@ -83,19 +86,19 @@ pub trait EndpointResult: Sized {
     /// `Self` for most types.
     type DecodeAs;
 
-    fn finish<FA>(&self, api: FA)
+    fn finish<FA>(&self)
     where
-        FA: ManagedTypeApi + EndpointFinishApi + Clone + 'static;
+        FA: ManagedTypeApi + EndpointFinishApi;
 }
 
-pub fn finish_all<FA, I, T>(api: FA, items: I)
+pub fn finish_all<FA, I, T>(items: I)
 where
-    FA: ManagedTypeApi + EndpointFinishApi + Clone + 'static,
+    FA: ManagedTypeApi + EndpointFinishApi,
     I: Iterator<Item = T>,
     T: EndpointResult,
 {
     for item in items {
-        item.finish(api.clone());
+        item.finish::<FA>();
     }
 }
 
@@ -106,20 +109,20 @@ where
 {
     type DecodeAs = Self;
 
-    fn finish<FA>(&self, api: FA)
+    fn finish<FA>(&self)
     where
-        FA: ManagedTypeApi + EndpointFinishApi + Clone + 'static,
+        FA: ManagedTypeApi + EndpointFinishApi,
     {
-        self.top_encode_or_exit(ApiOutputAdapter::new(api.clone()), api, finish_exit);
+        self.top_encode_or_exit(ApiOutputAdapter::<FA>::new(), (), finish_exit::<FA>);
     }
 }
 
 #[inline(always)]
-fn finish_exit<FA>(api: FA, encode_err: EncodeError) -> !
+fn finish_exit<FA>(_: (), encode_err: EncodeError) -> !
 where
     FA: ManagedTypeApi + EndpointFinishApi + ErrorApi + 'static,
 {
     let mut message_buffer = ManagedBuffer::<FA>::new_from_bytes(err_msg::FINISH_ENCODE_ERROR);
     message_buffer.append_bytes(encode_err.message_bytes());
-    api.signal_error_from_buffer(message_buffer.get_raw_handle())
+    FA::error_api_impl().signal_error_from_buffer(message_buffer.get_raw_handle())
 }
