@@ -5,25 +5,24 @@ use crate::{
 use elrond_codec::*;
 
 #[inline(always)]
-pub fn load_single_arg<AA, T>(api: AA, index: i32, arg_id: ArgId) -> T
+pub fn load_single_arg<AA, T>(index: i32, arg_id: ArgId) -> T
 where
     T: TopDecode,
     AA: ManagedTypeApi + EndpointArgumentApi + ErrorApi,
 {
     T::top_decode_or_exit(
-        ArgDecodeInput::new(api.clone(), index),
-        (api, arg_id),
-        load_single_arg_exit,
+        ArgDecodeInput::<AA>::new(index),
+        arg_id,
+        load_single_arg_exit::<AA>,
     )
 }
 
 #[inline(always)]
-fn load_single_arg_exit<AA>(ctx: (AA, ArgId), de_err: DecodeError) -> !
+fn load_single_arg_exit<AA>(arg_id: ArgId, de_err: DecodeError) -> !
 where
     AA: ManagedTypeApi + EndpointArgumentApi + ErrorApi,
 {
-    let (api, arg_id) = ctx;
-    signal_arg_de_error(api, arg_id, de_err)
+    signal_arg_de_error::<AA>(arg_id, de_err)
 }
 
 /// It's easier to generate code from macros using this function, instead of the DynArg method.
