@@ -9,6 +9,7 @@ mod esdt_token_type;
 mod h256;
 mod h256_address;
 mod queue;
+mod randomness_source;
 mod token_identifier;
 
 pub use boxed_bytes::BoxedBytes;
@@ -22,4 +23,5 @@ pub use esdt_token_type::EsdtTokenType;
 pub use h256::H256;
 pub use h256_address::Address;
 pub use queue::Queue;
+pub use randomness_source::RandomnessSource;
 pub use token_identifier::TokenIdentifier;
