@@ -1,8 +1,8 @@
 use super::ArgBuffer;
 use crate::{
-    api::{ErrorApi, Handle, ManagedTypeApi},
+    api::{ErrorApi, ErrorApiImpl, Handle, ManagedTypeApi},
     err_msg,
-    types::{ManagedBuffer, ManagedType, ManagedVec, ManagedVecIterator},
+    types::{ManagedBuffer, ManagedType, ManagedVec, ManagedVecRefIterator},
     DynArgOutput,
 };
 use alloc::vec::Vec;
@@ -12,16 +12,17 @@ use elrond_codec::{
 };
 
 #[derive(Debug)]
+#[repr(transparent)]
 pub struct ManagedArgBuffer<M>
 where
-    M: ManagedTypeApi + ErrorApi + 'static,
+    M: ManagedTypeApi + 'static,
 {
     pub(crate) data: ManagedVec<M, ManagedBuffer<M>>,
 }
 
 impl<M: ManagedTypeApi> ManagedType<M> for ManagedArgBuffer<M>
 where
-    M: ManagedTypeApi + ErrorApi + 'static,
+    M: ManagedTypeApi + 'static,
 {
     #[inline]
     fn from_raw_handle(handle: Handle) -> Self {
@@ -34,11 +35,15 @@ where
     fn get_raw_handle(&self) -> Handle {
         self.data.get_raw_handle()
     }
+
+    fn transmute_from_handle_ref(handle_ref: &Handle) -> &Self {
+        unsafe { core::mem::transmute(handle_ref) }
+    }
 }
 
 impl<M: ManagedTypeApi> ManagedArgBuffer<M>
 where
-    M: ManagedTypeApi + ErrorApi + 'static,
+    M: ManagedTypeApi + 'static,
 {
     #[inline]
     pub fn new_empty() -> Self {
@@ -67,9 +72,9 @@ where
     }
 }
 
-impl<M: ManagedTypeApi> ManagedArgBuffer<M>
+impl<M> ManagedArgBuffer<M>
 where
-    M: ManagedTypeApi + ErrorApi + 'static,
+    M: ManagedTypeApi + 'static,
 {
     #[inline]
     pub fn len(&self) -> usize {
@@ -86,18 +91,9 @@ where
         self.data.push(raw_arg);
     }
 
-    pub fn push_arg<T: TopEncode>(&mut self, arg: T) {
-        let mut encoded_buffer = ManagedBuffer::new();
-        arg.top_encode_or_exit(
-            &mut encoded_buffer,
-            M::instance(), // TODO: remove
-            managed_arg_buffer_push_exit,
-        );
-        self.push_arg_raw(encoded_buffer);
-    }
-
     /// Concatenates 2 managed arg buffers. Consumes both arguments in the process.
     #[inline]
+    #[must_use]
     pub fn concat(mut self, other: ManagedArgBuffer<M>) -> Self {
         self.data.append_vec(other.data);
         self
@@ -112,20 +108,31 @@ where
     }
 }
 
+impl<M> ManagedArgBuffer<M>
+where
+    M: ManagedTypeApi + ErrorApi + 'static,
+{
+    pub fn push_arg<T: TopEncode>(&mut self, arg: T) {
+        let mut encoded_buffer = ManagedBuffer::new();
+        arg.top_encode_or_exit(&mut encoded_buffer, (), managed_arg_buffer_push_exit::<M>);
+        self.push_arg_raw(encoded_buffer);
+    }
+}
+
 #[inline(always)]
-fn managed_arg_buffer_push_exit<A>(api: A, encode_err: EncodeError) -> !
+fn managed_arg_buffer_push_exit<A>(_: (), encode_err: EncodeError) -> !
 where
     A: ManagedTypeApi + ErrorApi + 'static,
 {
     let mut message_buffer =
         ManagedBuffer::<A>::new_from_bytes(err_msg::CONTRACT_CALL_ENCODE_ERROR);
     message_buffer.append_bytes(encode_err.message_bytes());
-    api.signal_error_from_buffer(message_buffer.get_raw_handle())
+    A::error_api_impl().signal_error_from_buffer(message_buffer.get_raw_handle())
 }
 
 impl<M: ManagedTypeApi> ManagedArgBuffer<M>
 where
-    M: ManagedTypeApi + ErrorApi + 'static,
+    M: ManagedTypeApi + 'static,
 {
     pub fn to_legacy_arg_buffer(&self) -> ArgBuffer {
         let mut result = ArgBuffer::new();
@@ -138,9 +145,9 @@ where
 
 impl<M: ManagedTypeApi> ManagedArgBuffer<M>
 where
-    M: ManagedTypeApi + ErrorApi + 'static,
+    M: ManagedTypeApi + 'static,
 {
-    pub fn raw_arg_iter(&self) -> ManagedVecIterator<M, ManagedBuffer<M>> {
+    pub fn raw_arg_iter(&self) -> ManagedVecRefIterator<M, ManagedBuffer<M>> {
         self.data.iter()
     }
 }
