@@ -99,12 +99,12 @@ where
     type DecodeAs = MultiArgVec<T::DecodeAs>;
 
     #[inline]
-    fn finish<FA>(&self, api: FA)
+    fn finish<FA>(&self)
     where
-        FA: ManagedTypeApi + EndpointFinishApi + Clone + 'static,
+        FA: ManagedTypeApi + EndpointFinishApi,
     {
         for elem in self.0.iter() {
-            elem.finish(api.clone());
+            elem.finish::<FA>();
         }
     }
 }
